package com.plugin;

import java.util.Arrays;

import org.gradle.api.Plugin;
import org.gradle.api.Project;
import org.gradle.api.plugins.JavaPlugin;
import org.gradle.api.plugins.JavaPluginConvention;
import org.gradle.api.tasks.SourceSet;
import org.gradle.api.tasks.testing.Test;

public class JavaApiTestPlugin implements Plugin<Project>
{
	@Override
	public void apply(Project project)
	{
		project.getPlugins().withType(JavaPlugin.class, javaPlugin -> {

			JavaPluginConvention javaConvention = project.getConvention().getPlugin(JavaPluginConvention.class);

			SourceSet main = javaConvention.getSourceSets().getByName(SourceSet.MAIN_SOURCE_SET_NAME);
			SourceSet test = javaConvention.getSourceSets().getByName(SourceSet.TEST_SOURCE_SET_NAME);

			final SourceSet testApiSourceSet = javaConvention.getSourceSets().create("testApi", sourceSet -> {
				sourceSet.setCompileClasspath(project.files(main.getOutput(), test.getOutput()));
				sourceSet.setRuntimeClasspath(project.files(main.getOutput(), test.getOutput()));
				sourceSet.getJava().srcDir(Arrays.asList("src/testApi/java"));
				sourceSet.setRuntimeClasspath(sourceSet.getOutput());
				sourceSet.getResources().srcDir("src/testApi/resources");
			});

			project.getTasks().create("apiTest", Test.class, testApi -> {
				testApi.setTestClassesDirs(testApiSourceSet.getOutput().getClassesDirs());
				testApi.setClasspath(testApiSourceSet.getRuntimeClasspath());

				testApi.useJUnitPlatform();
			});

			project.getDependencies().add("testApiImplementation", "org.testcontainers:testcontainers:1.7.1");
			project.getDependencies().add("testApiImplementation", "com.google.guava:guava:30.1.1-jre");
		});
	}
}
